@@ -19,7 +19,7 @@
  *
  * This file contains functions for interfacing with the GDB remote protocol.
  *
- * $Id: jtag.h 306 2012-11-05 13:18:17Z joerg_wunsch $
+ * $Id: jtag.h 388 2020-09-05 20:46:04Z joerg_wunsch $
  */
 
 #ifndef JTAG_H
@@ -157,6 +157,39 @@ typedef struct {
     unsigned char mcu_base_addr[2];	// IO space base address of MCU control
 } xmega_device_desc_type;
 
+// JTAGICE3 megaAVR parameter structure
+typedef struct {
+    unsigned char flash_page_size[2];   // in bytes
+    unsigned char flash_size[4];        // in bytes
+    unsigned char dummy1[4];            // always 0
+    unsigned char boot_address[4];      // maximal (BOOTSZ = 3) bootloader
+                                        // address, in 16-bit words (!)
+    unsigned char sram_offset[2];       // pointing behind IO registers
+    unsigned char eeprom_size[2];
+    unsigned char eeprom_page_size;
+    unsigned char ocd_revision;         // see XML; basically:
+                                        // t13*, t2313*, t4313:        0
+                                        // all other DW devices:       1
+                                        // ATmega128(A):               1 (!)
+                                        // ATmega16*,162,169*,32*,64*: 2
+                                        // ATmega2560/2561:            4
+                                        // all other megaAVR devices:  3
+    unsigned char always_one;           // always = 1
+    unsigned char allow_full_page_bitstream; // old AVRs, see XML
+    unsigned char dummy2[2];            // always 0
+                                        // all IO addresses below are given
+                                        // in IO number space (without
+                                        // offset 0x20), even though e.g.
+                                        // OSCCAL always resides outside
+    unsigned char idr_address;          // IDR, aka. OCDR
+    unsigned char eearh_address;        // EEPROM access
+    unsigned char eearl_address;
+    unsigned char eecr_address;
+    unsigned char eedr_address;
+    unsigned char spmcr_address;
+    unsigned char osccal_address;
+} jtag3_device_desc_type;
+
 #define fill_b4(u) \
 { ((u) & 0xffUL), (((u) & 0xff00UL) >> 8), \
   (((u) & 0xff0000UL) >> 16), (((u) & 0xff000000UL) >> 24) }
@@ -190,6 +223,9 @@ typedef struct {
                                        // fuse byte, so 0x00008000 for megaAVRs (Xmega
                                        // devices don't have OCDEN at all)
 
+    unsigned char osccal;              // OSCCAL offset (XML param)
+    unsigned char ocdrev;              // OCD revision (Studio 6 XML param)
+
     jtag1_device_desc_type dev_desc1;  // Device descriptor to download to
                                        // mkI device
     jtag2_device_desc_type dev_desc2;  // Device descriptor to download to
@@ -238,6 +274,7 @@ enum
 
     JTAG_RESPONSE_TIMEOUT	      = 1000000,
     JTAG_COMM_TIMEOUT		      = 100000,
+    JTAG3_PIPE_TIMEOUT                = 100,
 
     MAX_FLASH_PAGE_SIZE               = 512,
     MAX_EEPROM_PAGE_SIZE              = 32,
@@ -359,6 +396,11 @@ enum
     MESSAGE_START		= 0x1b,
     TOKEN			= 0x0e,
 
+    // On the JTAGICE3, the USB reader framework modifies the token of
+    // a received event into this so the upper layers can tell an event
+    // message apart from a normal response message.
+    TOKEN_EVT3			= 'T',
+
     // Max message size we are willing to accept.  Prevents us from trying
     // to allocate too much VM in case we received a nonsensical packet
     // length.  We have to allocate the buffer as soon as we've got the
@@ -547,12 +589,66 @@ enum
     PAR_ALLOW_PAGEPROGRAMMING_IN_SCANCHAIN = 0x24,
     PAGEPROG_NOT_ALLOWED	= 0x00,
     PAGEPROG_ALLOWED		= 0x01,
+
+    /* EDBG related constants */
+    EDBG_VENDOR_AVR_CMD		= 0x80,
+    EDBG_VENDOR_AVR_RSP		= 0x81,
+    EDBG_VENDOR_AVR_EVT		= 0x82,
+
 };
 
 enum {
     PC_INVALID			      = 0xffffffff
 };
 
+/*
+ * JTAG ICE mkII breakpoints are quite tricky.
+ *
+ * There are four possible breakpoint slots in JTAG.  The first one is
+ * always reserved for single-stepping, and cannot be accessed
+ * directly.  The second slot (ID #1) can be freely used as a code
+ * breakpoint (only).  The third and fourth slot can either be used as
+ * a code breakpoint, as an independent (byte-wide) data breakpoint
+ * (i.e. a watchpoint in GDB terms), or together as a data breakpoint
+ * consisting of an address and a mask.  The latter does not match
+ * directly to GDB watchpoints (of a certain lenght > 1) but imposes
+ * the additional requirement that the base address be aligned
+ * properly wrt. the mask.
+ *
+ * The single-step breakpoint can indirectly be used by filling the
+ * respective high-level language information into the event memory,
+ * and issuing a high-level "step over" single-step.  As GDB does not
+ * install the breakpoints for "stepi"-style single-steps (which would
+ * require the very same JTAG breakpoint register), this ought to
+ * work.
+ *
+ * Finally, there are software breakpoints where the respective
+ * instruction will be replaced by a BREAK instruction in flash ROM by
+ * means of an SPM call.  Some devices do not allow for this (as they
+ * are said to be broken), and in general as this method contributes
+ * to flash wear, we rather do not uninstall and reinstall these
+ * breakpoints each time GDB asks for it, but instead "cache" them
+ * locally, and only delete and add them as they actually change.
+ *
+ * To add to the mess, GDB's remote protocol isn't very smart about
+ * telling us whether the next list of breakpoints + resume are
+ * actually meant to be a high-level language (HLL) single-step
+ * command.  Thus, always assume the last breakpoint that comes in to
+ * be a single-step breakpoint, and replace the "resume" operation by
+ * a "step" one in that case.  At least, GDB issues the breakpoint
+ * list in the order of breakpoint numbers, so any possible HLL
+ * single-step breakoint must be the last one in the list.
+ *
+ * Finally, GDB has explicit commands for setting hardware-assisted
+ * breakpoints, but the default "break" command uses a software
+ * breakpoint.  We try to replace as many software breakpoints as
+ * possible by hardware breakpoints though, for the sake of
+ * efficiency, yet would want to respect the user's choice for setting
+ * a hardware breakpoint ("hbreak" or "thbreak")...
+ *
+ * XXX This is not done yet.
+ */
+
 enum bpType
 {
     NONE,           // disabled.
@@ -563,6 +659,54 @@ enum bpType
     DATA_MASK,      // mask for data space breakpoint.
 };
 
+enum {
+  // We distinguish the total possible breakpoints and those for each type
+  // (code or data) - see above
+  MAX_BREAKPOINTS2_CODE = 4,
+  MAX_BREAKPOINTS2_DATA = 2,
+  MAX_BREAKPOINTS2 = 4,
+  // various slot #s
+  BREAKPOINT2_XMEGA_UNAVAIL = 1,
+  BREAKPOINT2_FIRST_DATA = 2,
+  BREAKPOINT2_DATA_MASK = 3,
+
+  MAX_TOTAL_BREAKPOINTS2 = 255
+};
+
+struct breakpoint2
+{
+    // High-level information on breakpoint
+    unsigned int address;
+    unsigned int mask_pointer;
+    bpType type;
+    bool enabled;
+
+    // Used to flag end of list
+    bool last;
+
+    // Low-level information on breakpoint
+    bool icestatus; // Status of breakpoint in ICE itself: 'true'
+                    // when is enabled in ACTUAL device
+    bool toremove;  // Delete this guy in ICE
+    bool toadd;     // Add this guy in ICE
+    uchar bpnum;    // ICE's breakpoint number (0x00 for software)
+    bool has_mask;  // data watchpoint has a mask associated
+};
+
+const struct breakpoint2 default_bp =
+{
+    0,				/* address */
+    0,				/* mask_pointer */
+    NONE,			/* type */
+    false,			/* enabled */
+    true,			/* last */
+    false,			/* icestatus */
+    false,			/* toremove */
+    false,			/* toadd */
+    0,				/* bpnum*/
+    false,			/* has_mask */
+};
+
 // Enumerations for target memory type.
 typedef enum {
     MEM_FLASH = 0,
@@ -576,6 +720,8 @@ extern const int BFDmemorySpaceOffset[];
 enum emulator {
   EMULATOR_JTAGICE,
   EMULATOR_DRAGON,
+  EMULATOR_JTAGICE3,
+  EMULATOR_EDBG
 };
 
 
@@ -605,6 +751,10 @@ typedef struct {
 // The Sync_CRC/EOP message terminator (no real CRC in sight...)
 #define JTAG_EOM 0x20, 0x20
 
+enum debugproto {
+    PROTO_JTAG, PROTO_DW, PROTO_PDI,
+};
+
 class jtag
 {
   protected:
@@ -618,15 +768,25 @@ class jtag
   // For the mkII device, is the box attached via USB?
   bool is_usb;
 
-  // A control pipe to talk to the USB daemon.
-  int ctrlPipe;
-
   // The type of our emulator: JTAG ICE, or AVR Dragon.
   emulator emu_type;
 
   // Whether nSRST is to be applied when connecting (override JTD bit).
   bool apply_nSRST;
 
+  // Total breakpoints including software
+  breakpoint2 bp[MAX_TOTAL_BREAKPOINTS2];
+
+  // Xmega hard breakpoing break handling
+  unsigned int xmega_n_bps;
+  unsigned long xmega_bps[2];
+
+  // This device or connection cannot handle hard BPs
+  bool softbp_only;
+
+  // Target device is an ATxmega one
+  bool is_xmega;
+
   public:
   // Whether we are in "programming mode" (changes how program memory
   // is written, apparently)
@@ -647,7 +807,8 @@ class jtag
   } dchain;
 
   protected:
-  pid_t openUSB(const char *jtagDeviceName);
+  void openUSB(const char *jtagDeviceName);
+  void resetUSB(void);
   int safewrite(const void *b, int count);
   void changeLocalBitRate(int newBitRate);
   void restoreSerialPort(void);
@@ -711,10 +872,12 @@ class jtag
   virtual void deleteAllBreakpoints(void) = 0;
 
   /** Delete breakpoint at the specified address. */
-  virtual bool deleteBreakpoint(unsigned int address, bpType type, unsigned int length) = 0;
+  bool deleteBreakpoint(unsigned int address, bpType type, unsigned int length);
 
   /** Add a code breakpoint at the specified address. */
-  virtual bool addBreakpoint(unsigned int address, bpType type, unsigned int length) = 0;
+  bool addBreakpoint(unsigned int address, bpType type, unsigned int length);
+
+  bool layoutBreakpoints(void);
 
   /** Send the breakpoint details down to the JTAG box. */
   virtual void updateBreakpoints(void) = 0;
@@ -722,12 +885,6 @@ class jtag
   /** True if there is a breakpoint at address */
   virtual bool codeBreakpointAt(unsigned int address) = 0;
 
-  /** True if there is a breakpoint between start (inclusive) and 
-      end (exclusive) */
-  virtual bool codeBreakpointBetween(unsigned int start, unsigned int end) = 0;
-
-  virtual void stopAt(unsigned int address) = 0;
-
   /** Parse a list of event names to *not* cause a break. */
   virtual void parseEvents(const char *) = 0;
 
@@ -894,6 +1051,22 @@ class jtag_timeout_exception: public jtag_exception
     jtag_timeout_exception(): jtag_exception("JTAG ICE timeout exception") {}
 };
 
-extern struct jtag *theJtagICE;
+class jtag_io_exception: public jtag_exception
+{
+  protected:
+    unsigned int response_code;
+
+  public:
+    jtag_io_exception(): jtag_exception("Unknown JTAG response exception")
+    {
+        response_code = 0;
+    }
+    jtag_io_exception(unsigned int code);
+
+    unsigned int get_response(void) { return response_code; }
+};
+
+
+extern class jtag *theJtagICE;
 
 #endif
