@@ -21,11 +21,12 @@
  *
  * This file contains the main() & support for avrjtagd.
  *
- * $Id: main.cc 301 2012-11-02 21:51:18Z joerg_wunsch $
+ * $Id: main.cc 388 2020-09-05 20:46:04Z joerg_wunsch $
  */
 
 
 #include <ctype.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,6 +45,7 @@
 #include "jtag.h"
 #include "jtag1.h"
 #include "jtag2.h"
+#include "jtag3.h"
 #include "gnu_getopt.h"
 
 bool ignoreInterrupts;
@@ -69,7 +71,11 @@ static int makeSocket(struct sockaddr_in *name)
         throw jtag_exception();
 
     if (bind(sock, (struct sockaddr *)name, sizeof(*name)) < 0)
-        throw jtag_exception();
+    {
+	if (errno == EADDRINUSE)
+	    fprintf(stderr, "bind() failed: another server is still running on this port\n");
+        throw jtag_exception("bind() failed");
+    }
 
     protoent = getprotobyname("tcp");
     if (protoent == NULL)
@@ -153,6 +159,10 @@ static void usage(const char *progname)
 	    "  -1, --mkI                   Connect to JTAG ICE mkI (default)\n");
     fprintf(stderr,
 	    "  -2, --mkII                  Connect to JTAG ICE mkII\n");
+    fprintf(stderr,
+	    "  -3, --jtag3                 Connect to JTAGICE3 (Firmware 2.x)\n");
+    fprintf(stderr,
+            "  -4, --edbg                  Atmel-ICE, or JTAGICE3 (firmware 3.x), or EDBG Integrated Debugger\n");
     fprintf(stderr,
             "  -B, --jtag-bitrate <rate>   Set the bitrate that the JTAG box communicates\n"
             "                                with the avr target device. This must be less\n"
@@ -236,8 +246,14 @@ static void usage(const char *progname)
 	    "HOST_NAME defaults to 0.0.0.0 (listen on any interface).\n"
 	    "\":PORT\" is required to put avarice into gdb server mode.\n\n");
     fprintf(stderr,
-	    "e.g. %s --erase --program --file test.bin --jtag /dev/ttyS0 :4242\n\n",
-	    progname);
+            "Example usage:\n");
+    fprintf(stderr,
+            "\t%s --jtag /dev/ttyS0 :4242\n",
+            progname);
+    fprintf(stderr,
+            "\t%s --dragon :4242\n",
+            progname);
+    fprintf(stderr, "\n");
     exit(0);
 }
 
@@ -295,6 +311,8 @@ static struct option long_opts[] = {
     /* name,                 has_arg, flag,   val */
     { "mkI",                 0,       0,     '1' },
     { "mkII",                0,       0,     '2' },
+    { "jtag3",               0,       0,     '3' },
+    { "edbg",                0,       0,     '4' },
     { "jtag-bitrate",        1,       0,     'B' },
     { "capture",             0,       0,     'C' },
     { "daisy-chain",         1,       0,     'c' },
@@ -349,13 +367,13 @@ int main(int argc, char **argv)
     bool detach = false;
     bool capture = false;
     bool verify = false;
-    bool is_dragon = false;
     bool apply_nsrst = false;
     bool is_xmega = false;
     char *progname = argv[0];
     enum {
-	MKI, MKII, MKII_DW, MKII_PDI
-    } protocol = MKI;		// default to mkI protocol
+	MKI, MKII, DRAGON, JTAG3, EDBG
+    } devicetype = MKI;		// default to mkI devicetype
+    enum debugproto proto = PROTO_JTAG;
     int  option_index;
     unsigned int units_before = 0;
     unsigned int units_after = 0;
@@ -371,7 +389,7 @@ int main(int argc, char **argv)
 
     while (1)
     {
-        int c = getopt_long (argc, argv, "12B:Cc:DdeE:f:ghIj:kL:lP:pRrVvwW:xX",
+        int c = getopt_long (argc, argv, "1234B:Cc:DdeE:f:ghIj:kL:lP:pRrVvwW:xX",
                              long_opts, &option_index);
         if (c == -1)
             break;              /* no more options */
@@ -383,13 +401,17 @@ int main(int argc, char **argv)
             case 'k':
                 knownParts();
 	    case '1':
-		protocol = MKI;
+		devicetype = MKI;
 		break;
 	    case '2':
-		// If we've already seen a -w option, don't revert to -2.
-		if (protocol != MKII_DW && protocol != MKII_PDI)
-		    protocol = MKII;
+		devicetype = MKII;
 		break;
+	    case '3':
+		devicetype = JTAG3;
+		break;
+            case '4':
+                devicetype = EDBG;
+                break;
             case 'B':
 		jtagBitrate = parseJtagBitrate(optarg);
                 break;
@@ -426,10 +448,7 @@ int main(int argc, char **argv)
                 inFileName = optarg;
                 break;
 	    case 'g':
-		// If we've already seen a -w option, don't revert to -2.
-		if (protocol != MKII_DW && protocol != MKII_PDI)
-		    protocol = MKII;
-	        is_dragon = true;
+		devicetype = DRAGON;
 		break;
             case 'I':
                 ignoreInterrupts = true;
@@ -462,7 +481,7 @@ int main(int argc, char **argv)
                 verify = true;
                 break;
             case 'w':
-	        protocol = MKII_DW;
+		proto = PROTO_DW;
 		break;
             case 'W':
                 fuses = optarg;
@@ -473,7 +492,7 @@ int main(int argc, char **argv)
                 break;
             case 'X':
                 is_xmega = true;
-                protocol = MKII_PDI;
+		proto = PROTO_PDI;
                 break;
             default:
                 fprintf (stderr, "getop() did something screwey");
@@ -537,7 +556,7 @@ int main(int argc, char **argv)
         usage (progname);
     }
 
-    if (jtagBitrate == 0 && (protocol == MKI || protocol == MKII))
+    if (jtagBitrate == 0 && (proto == PROTO_JTAG))
     {
         fprintf (stdout,
                  "Defaulting JTAG bitrate to 250 kHz.\n\n");
@@ -555,34 +574,38 @@ int main(int argc, char **argv)
 
       if (cp != NULL)
 	jtagDeviceName = cp;
-      else if (is_dragon)
+      else if (devicetype == DRAGON || devicetype == JTAG3 || devicetype == EDBG)
 	jtagDeviceName = "usb";
       else
 	jtagDeviceName = "/dev/avrjtag";
     }
 
+    if (debugMode)
+      setvbuf(stderr, NULL, _IOLBF, 0);
+
     int rv = 0;			// return value from main()
 
     try {
 	// And say hello to the JTAG box
-	switch (protocol) {
+	switch (devicetype) {
 	case MKI:
 	    theJtagICE = new jtag1(jtagDeviceName, device_name, apply_nsrst);
 	    break;
 
 	case MKII:
-	    theJtagICE = new jtag2(jtagDeviceName, device_name, PROTO_JTAG,
-				   is_dragon, apply_nsrst, is_xmega);
+	case DRAGON:
+	    theJtagICE = new jtag2(jtagDeviceName, device_name, proto,
+				   devicetype == DRAGON, apply_nsrst, is_xmega);
 	    break;
 
-	case MKII_DW:
-	    theJtagICE = new jtag2(jtagDeviceName, device_name, PROTO_DW,
-				   is_dragon, apply_nsrst);
+	case JTAG3:
+	    theJtagICE = new jtag3(jtagDeviceName, device_name, proto,
+				   apply_nsrst, is_xmega);
 	    break;
 
-	case MKII_PDI:
-	    theJtagICE = new jtag2(jtagDeviceName, device_name, PROTO_PDI,
-				   is_dragon, apply_nsrst, true);
+	case EDBG:
+	    theJtagICE = new jtag3(jtagDeviceName, device_name, proto,
+				   apply_nsrst, is_xmega, true);
 	    break;
 	}
 
@@ -600,7 +623,7 @@ int main(int argc, char **argv)
 
         if (erase)
         {
-            if (protocol == MKII_DW)
+            if (proto == PROTO_DW)
             {
                 statusOut("WARNING: Chip erase not possible in debugWire mode; ignored\n");
             }
